@@ -871,15 +871,6 @@ static int start_fn(void *arg)
 	strcpy(current->comm, "UNICORN-");
 	strncat(current->comm, k->name, 4);
 
-	if (k->priority > XPRIO_BACKGRND_APPL) {
-		int adj = (k->priority-XPRIO_BACKGRND_APPL) / 20;
-#ifdef DEF_NICE
-		current->nice -= adj;
-#else
-		// Preemptive kernel (2.6 for example)
-		set_user_nice(k->thread,-adj);
-#endif
-	}
 	unlock_kernel();
 
 	DBG(RAPI_D,"start %.4s\n",k->name);
@@ -926,9 +917,12 @@ DWORD xt_start(
 		return FAILURE;
 	}
 #ifdef CONFIG_SMP
+#ifndef CPU_MASK_CPU0
+#define CPU_MASK_CPU0 1UL
+#endif
 	// Hack to try to make it work on SMP !!!!
 	// Lock the threads to CPU #0
-	set_cpus_allowed(k->thread,1UL << 0);
+	set_cpus_allowed(k->thread,CPU_MASK_CPU0);
 #endif
 	return SUCCESS;	
 }
