@@ -10,6 +10,8 @@ extern "C" {
 	const char *get_modem_event_string(ADSL_DEVICE *device);
 	const char *get_modem_failure_string(ADSL_DEVICE *device);
 	const char *get_time_connected_string(ADSL_DEVICE *device);
+	unsigned long long get_cells_rcvd(ADSL_DEVICE *device);
+	unsigned long long get_cells_sent(ADSL_DEVICE *device);
 	const char *get_modulation_string(ADSL_DEVICE *device);
 	int get_us_rate(ADSL_DEVICE *device);
 	int get_ds_rate(ADSL_DEVICE *device);
