@@ -20,6 +20,7 @@
 #include <linux/atm.h>
 #include <linux/atmdev.h>
 #include <linux/skbuff.h>
+#include <asm/uaccess.h>
 #include "types.h"
 #include "amu/amas.h"
 #include "crc.h"
@@ -77,9 +78,6 @@ typedef struct unicorn_atmdrv {
 	struct unicorn_entrypoints *ep;
 } unicorn_atmdrv_t;
 
-// use the dev_data pointer to store the reassembly buffer when receiving
-#define VCC_REAS_BUF(vcc) (((struct sk_buff *) (vcc)->dev_data))
- 
 // extended skb_data structure used when encoding cells
 struct atm_ext_skb_data {
 	struct atm_skb_data atm_skb_data;
@@ -689,13 +687,14 @@ static struct sk_buff *rawcell_decode(struct atm_vcc *vcc, unsigned char *cell)
 	DUMP_PACKET(DATA_D,cell,ATM_CELL_SIZE);
 
 	// get reassembly buffer
-	skb = VCC_REAS_BUF(vcc);
+	skb = vcc->dev_data;
 	if (!skb) {
-		skb = VCC_REAS_BUF(vcc) = dev_alloc_skb(max_mru);
+		skb = dev_alloc_skb(max_mru);
 		if (!skb) {
 			WARN("dev_alloc_skb failed\n");
 			return NULL;
 		}
+		vcc->dev_data = skb;
 	}
     
 	// copy data
@@ -708,9 +707,9 @@ static struct sk_buff *rawcell_decode(struct atm_vcc *vcc, unsigned char *cell)
   	// check for end of buffer
   	if (cell[3] & 0x2) {
 		// the aal5 buffer ends here, cut the buffer. 		
-		VCC_REAS_BUF(vcc) = NULL;
+		vcc->dev_data = NULL;
 	} else {
-		VCC_REAS_BUF(vcc) = skb;
+		vcc->dev_data = skb;
 		skb = NULL;
 	}
 	return skb;
@@ -992,7 +991,7 @@ static int unicorn_atm_open(struct atm_vcc *vcc)
 		return -EINVAL;
 	}
 
-	VCC_REAS_BUF(vcc) = NULL;
+	vcc->dev_data = NULL;
 
 	set_bit(ATM_VF_ADDR,    &vcc->flags);
 	set_bit(ATM_VF_PARTIAL, &vcc->flags);
@@ -1037,9 +1036,9 @@ static void unicorn_atm_close(struct atm_vcc *vcc)
 	DBG(ATM_D,"\n");
 
 	// freeing resources
-	if (VCC_REAS_BUF(vcc)) {
-		dev_kfree_skb_any(VCC_REAS_BUF(vcc));
-		VCC_REAS_BUF(vcc) = NULL;
+	if (vcc->dev_data) {
+		dev_kfree_skb_any(vcc->dev_data);
+		vcc->dev_data = NULL;
 	}
 	clear_bit(ATM_VF_PARTIAL, &vcc->flags);
 
