@@ -14,6 +14,7 @@
 #include <linux/pci.h>
 #include <linux/interrupt.h>
 #include <asm/io.h>
+#include <asm/uaccess.h>
 #include <linux/module.h>
 #include <linux/delay.h>
 #include <asm/byteorder.h>
@@ -121,6 +122,7 @@ extern unsigned long highCarrierOff;
 extern unsigned long decreaseHighCarrier;
 extern unsigned long _boostPowerGdmt_;
 extern unsigned long g_ModemState;
+extern unsigned long _trellisact_;
 
 unsigned long AutoActivation=1;
 unsigned long LoopbackMode=0;
@@ -293,9 +295,7 @@ static void send_atm_init(struct unicorn_dev *dev,
 	send_atm->send_pending = 0;
 	send_atm->started = FALSE;
 	send_atm->cell_count = 0;
-#if LINUX
 	send_atm->lock = SPIN_LOCK_UNLOCKED;
-#endif
 }
 
 //----------------------------------------------------------------------
@@ -330,9 +330,7 @@ static void recv_atm_init(struct unicorn_dev *dev,
 		recv_atm->bufs[i] = buffer+i*size;
 		memset(recv_atm->bufs[i],0,size);
 	}
-#if LINUX
 	recv_atm->lock = SPIN_LOCK_UNLOCKED;
-#endif
 }
 
 //----------------------------------------------------------------------
@@ -353,9 +351,7 @@ static void atm_send_complete(struct unicorn_dev *dev)
 
 	DBG(RW_D,"\n");
 
-#if LINUX
 	spin_lock_irqsave(&send_atm->lock, flags);
-#endif
 	send_atm->send_pending--;
 	t = send_atm->turn_send;
 	if (test_and_clear_bit(0,&send_atm->busy[t])) {
@@ -363,9 +359,7 @@ static void atm_send_complete(struct unicorn_dev *dev)
 		t = (t+1)&(ATM_WRITES-1);
 		send_atm->turn_send = t;
 	}
-#if LINUX
 	spin_unlock_irqrestore(&send_atm->lock, flags);
-#endif
 }
 
 //----------------------------------------------------------------------
@@ -425,9 +419,7 @@ static void atm_recv_complete(struct unicorn_dev *dev)
 
 	DBG(RW_D,"\n");
 
-#if LINUX
 	spin_lock_irqsave(&recv_atm->lock, flags);
-#endif
 
 	// point to the next DMA buffer
 	recv_atm->turn_recv = (recv_atm->turn_recv+1)&(ATM_READS-1);
@@ -442,9 +434,8 @@ static void atm_recv_complete(struct unicorn_dev *dev)
 		recv_atm->turn_read = (recv_atm->turn_recv+1)&(ATM_READS-1);
 		recv_atm->bufofs = 0;
 	}
-#if LINUX
 	spin_unlock_irqrestore(&recv_atm->lock, flags);
-#endif
+
 	// Start the DMA
 	StartAtmDsXfer(dev,recv_atm->turn_recv);
 }
@@ -1931,9 +1922,7 @@ int unicorn_start_transmit(struct unicorn_dev *dev)
 
 	DBG(RW_D,"\n");
 
-#if LINUX
 	spin_lock_irqsave(&send_atm->lock, flags);
-#endif
 	
 	t = send_atm->turn_write;
 	n = send_atm->bufofs;
@@ -1943,15 +1932,11 @@ int unicorn_start_transmit(struct unicorn_dev *dev)
 		send_atm->bufofs = 0;
 		set_bit(0,&send_atm->busy[t]);
 		send_atm->send_pending++;
-#if LINUX
 		spin_unlock_irqrestore(&send_atm->lock, flags);
-#endif
 		StartAtmUsXfer(dev,t,send_atm->bufs[t],n);
 		return n;
 	}
-#if LINUX
 	spin_unlock_irqrestore(&send_atm->lock, flags);
-#endif
 	return 0;
 }
 
@@ -1967,9 +1952,7 @@ unsigned char *unicorn_rcv_getcell(struct unicorn_dev *dev)
 	PDWORD pf;
 	unsigned long flags;
 
-#if LINUX
 	spin_lock_irqsave(&recv_atm->lock, flags);
-#endif
 
 	t = recv_atm->turn_read;
 	p = recv_atm->bufs[t] + recv_atm->bufofs;
@@ -1987,9 +1970,7 @@ unsigned char *unicorn_rcv_getcell(struct unicorn_dev *dev)
 		p = NULL;
 	}
 
-#if LINUX
 	spin_unlock_irqrestore(&recv_atm->lock, flags);
-#endif
 	return p;
 }
 
@@ -2125,6 +2106,8 @@ int unicorn_msw_control(struct unicorn_dev *dev,T_MswCtrl *ctrl)
 			}
 			info->Report = last_event;
 			info->Failure = last_failure;
+			info->RcvdCells = dev->recv_atm.cell_count;
+			info->SentCells = dev->send_atm.cell_count;
 		}
 		break;
 	case MSW_CTL_SET_DEBUG_LEVEL:
@@ -2233,7 +2216,7 @@ unicorn_pci_init(void)
 	}
 	INFO("AFE %ld\n",useAFE);
 
-	pilotRealloc=0;
+	pilotRealloc=1;
 	_newCToneDetection_ = 1;
 	setINITIALDAC = 93;
 	if (useAFE == 70134) { 
@@ -2267,8 +2250,9 @@ unicorn_pci_init(void)
 	useRFCFixedRate = 1;
 	TrainingDelay = 120;
 	ExchangeDelay = 20;
+	_trellisact_ = 1;
 
-	//Interoperability=1;
+	Interoperability=1;
 
 	INFO("v %d.%d.%d, " __TIME__ " " __DATE__"\n",
 	     (VERS>>8)&0xf,(VERS>>4)&0xf,VERS&0xf);
