@@ -14,6 +14,7 @@
 #include <linux/init.h>
 #include <linux/usb.h>
 #include <asm/io.h>
+#include <asm/uaccess.h>
 #include <linux/delay.h>
 #include <linux/atmdev.h>
 #include <asm/byteorder.h>
@@ -119,8 +120,8 @@ static void dump_urb(struct urb *urb)
 
 	printk(KERN_DEBUG "hcpriv=%p,dev=%p,pipe=%x,status=%d,transfer_flags=%d\n",
 	       urb->hcpriv,urb->dev,urb->pipe,urb->status,urb->transfer_flags);
-	printk(KERN_DEBUG "bandwidth=%d,start_frame=%u,interval=%d,error_count=%d,timeout=%d\n",
-	    urb->bandwidth,urb->start_frame,urb->interval,urb->error_count,urb->timeout);
+	printk(KERN_DEBUG "bandwidth=%d,start_frame=%u,interval=%d,error_count=%d\n",
+	    urb->bandwidth,urb->start_frame,urb->interval,urb->error_count);
 	// buffers
 	printk(KERN_DEBUG "transfer_buffer=%p,transfer_buffer_length=%d,actual_length=%d\n",
 	       urb->transfer_buffer,urb->transfer_buffer_length,urb->actual_length);
@@ -145,11 +146,9 @@ static void fill_isoc_urb(struct urb *urb, struct usb_device *dev,
 	urb->pipe=pipe;
 	urb->transfer_buffer=buf;
 	urb->transfer_buffer_length=length;
-	urb->actual_length = 0;
 	urb->transfer_flags=USB_ISO_ASAP;
 	urb->start_frame = -1;
 	urb->interval = 1;
-	urb->timeout = 0;
 	urb->complete=complete;
 	urb->context=context;
 	{
@@ -415,7 +414,9 @@ static void StartAtmUsXfer(struct unicorn_dev *dev,int turn,unsigned char *buffe
 	if (dev->alternate_setting != 2) {
 		// Fill the isochronous URB
 		fill_isoc_urb(urb, dev->usb_dev, usb_sndisocpipe(dev->usb_dev,EP_ATM_ISO_OUT),
-			      buffer, length,  dev->usb_dev->epmaxpacketout[EP_ATM_ISO_OUT], 
+			      buffer, length,  
+			      usb_maxpacket(dev->usb_dev,
+					    usb_sndisocpipe(dev->usb_dev,EP_ATM_ISO_OUT),1), 
 			      AtmUsXferComplete, dev);
 	} else {
 		// Fill the bulk URB
@@ -444,6 +445,7 @@ static void AtmDsXferComplete(struct urb *urb)
 	    urb->status,urb->transfer_buffer_length,urb->actual_length);	
 
 	if (GlobalRemove) return;
+	if (!recv_atm->started) return;
 
 	if ((urb->status == USB_ST_DATAUNDERRUN) && (urb->actual_length > 0)) urb->status = 0;
 	if (urb->status) {
@@ -529,16 +531,16 @@ static void atm_start_rcv(struct unicorn_dev *dev)
 		// ISOC
 		size = ATM_DS_CELLS_PER_PKT*USB_CELL_LENGTH*ATM_DS_ISO_PACKETS;
 		recv_atm->num_reads = ATM_READS;
+		packet_size = usb_maxpacket(dev->usb_dev,usb_rcvisocpipe(dev->usb_dev,EP_ATM_ISO_IN),0);
 	} else {
 		// BULK
 		size = ATM_DS_CELLS_PER_PKT*USB_CELL_LENGTH*1;
 		recv_atm->num_reads = ATM_READS;
+		packet_size = 0;
 	}
 	recv_atm->maxlen = size;
 	recv_atm->cell_count = 0;
 
-	packet_size =  dev->usb_dev->epmaxpacketin[EP_ATM_ISO_IN];
-
 	DBG(1,"buffer=%p,size=%d,packet_size=%d,num_reads=%d\n",
 	    buffer,size,packet_size,recv_atm->num_reads);
 
@@ -611,171 +613,6 @@ static void atm_start_snd(struct unicorn_dev *dev)
 	send_atm->started = TRUE;
 }
 
-//----------------------------------------------------------------------
-// set_packetsize
-//----------------------------------------------------------------------
-static void set_packetsize(struct unicorn_dev *dev,int ea,int packet_size)
-{
-	struct usb_device *usb_dev =  dev->usb_dev;	
-	struct usb_interface *ifp;
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
-	struct usb_host_interface *if_desc;
-#else
-	struct usb_interface_descriptor *if_desc;
-#endif	
-	struct usb_endpoint_descriptor *ep_desc;	
-	int ep_num;
-
-	ifp = usb_ifnum_to_if(usb_dev,0);
-	if (!ifp) {
-		WARN(" usb_ifnum_to_if failed\n");
-		return;
-	}
-	dev->alternate_setting=ALT_SETTING;;
-	if_desc = &ifp->altsetting[dev->alternate_setting];
-	if (!if_desc) {
-		WARN("usb_interface_descriptor not found\n");
-		return;
-	}
-
-	ep_num = (ea & USB_ENDPOINT_NUMBER_MASK)-1;
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
-	ep_desc = &if_desc->endpoint[ep_num].desc;
-#else
-	ep_desc = &if_desc->endpoint[ep_num];
-#endif
-	if (!ep_desc) {
-		WARN("usb_endpoint_descriptor not found\n");
-	}
-	
-	DBG(1,"EP %02x,change MPS from %d to %d\n",
-	    ea,ep_desc->wMaxPacketSize,packet_size);
-	ep_desc->wMaxPacketSize = packet_size;
-	if (ea & USB_DIR_IN) {
-		usb_dev->epmaxpacketin[ea & USB_ENDPOINT_NUMBER_MASK] = ep_desc->wMaxPacketSize;
-	} else {
-		usb_dev->epmaxpacketout[ea & USB_ENDPOINT_NUMBER_MASK] = ep_desc->wMaxPacketSize;
-	}
-}
-
-//----------------------------------------------------------------------
-// get_ds_packetsize
-//----------------------------------------------------------------------
-static int get_ds_packetsize(unsigned short ds_rate)
-{
-    int max_packetsize;
-
-    // Downstream ATM pipe
-    if (ds_rate > 4000)			// > 1696kbs
-    {
-        max_packetsize = 8*USB_CELL_LENGTH;
-    }	
-    else if (ds_rate > 2339)		// > 992kbps
-    {
-        max_packetsize = 4*USB_CELL_LENGTH;
-    }
-    else if (ds_rate > 1584)		// > 672kbps
-    {
-        max_packetsize = 3*USB_CELL_LENGTH;
-    }
-    else if (ds_rate > 766)		// > 320kbps
-    {
-        max_packetsize = 2*USB_CELL_LENGTH;
-    }
-    else if (ds_rate > 0)		// <= 320kbps
-    {
-        max_packetsize = USB_CELL_LENGTH;
-    }
-    else
-    {
-        max_packetsize = 0;
-    }
-    return max_packetsize;
-}
-
-//----------------------------------------------------------------------
-// get_us_packetsize
-//----------------------------------------------------------------------
-static int get_us_packetsize(unsigned short us_rate)
-{
-    int max_packetsize;
-
-    // Upstream ATM pipe
-    if (us_rate > 1000)			// > 424kbps
-    {
-        max_packetsize  = 2*USB_CELL_LENGTH;
-    }
-    else if (us_rate > 0)		// <= 424kbps
-    {
-        max_packetsize  = USB_CELL_LENGTH;
-    }
-    else
-    {
-        max_packetsize = 0;
-    }
-    return max_packetsize;
-}
-
-//----------------------------------------------------------------------
-// adjust_bandwidth()
-//----------------------------------------------------------------------
-static void adjust_bandwidth(struct unicorn_dev *dev,unsigned short us_rate,unsigned short ds_rate)
-{
-
-#if (ALT_SETTING==4)
-	{
-		struct usb_device *usb_dev = dev->usb_dev;
-		int status;
-
-		// Change alt setting on the chip if bandwidth exceeds the ALT 4 Ds bandwidth.
-		// With alt setting 1, there can be up to 8 cells in the ds FIFO.
-		if (get_ds_packetsize(adsl_ds_cellrate) > 4*USB_CELL_LENGTH) {
-			if (dev->alternate_setting != 1) {
-				DBG(1,"ATM Ds MPS=%d,change from alternate setting %d to %d\n",
-				    get_ds_packetsize(adsl_ds_cellrate),dev->alternate_setting,1);
-				
-				dev->alternate_setting = 1;
-				status = usb_control_msg(usb_dev, usb_sndctrlpipe(usb_dev, 0),
-							 USB_REQ_SET_INTERFACE, USB_RECIP_INTERFACE, dev->alternate_setting,
-							 0, NULL, 0, HZ * 5);
-				if (status) {
-					WARN("usb_control_msg failed,status=%d\n",status);
-				}
-			}
-		} else {
-			if (dev->alternate_setting != 4) {
-				DBG(1,"ATM Ds MPS=%d,change from alternate setting %d to %d\n",
-				    get_ds_packetsize(adsl_ds_cellrate),dev->alternate_setting,4);
-				
-				dev->alternate_setting = 4;
-				status = usb_control_msg(usb_dev, usb_sndctrlpipe(usb_dev, 0),
-							 USB_REQ_SET_INTERFACE, USB_RECIP_INTERFACE, dev->alternate_setting,
-							 0, NULL, 0, HZ * 5);
-				if (status) {
-					WARN("usb_control_msg failed,status=%d\n",status);
-				}
-			}
-		}
-	}
-#endif
-
-    if (adsl_ds_cellrate > 0) {
-	    // Showtime
-	    if (!ep2Enable) {
-		    set_packetsize(dev,EP_OBC_ISO_OUT,0);
-	    }
-	    if (!ep3Enable) {
-		    set_packetsize(dev,EP_OBC_ISO_IN | USB_DIR_IN,0);
-	    }
-	    set_packetsize(dev,EP_ATM_ISO_OUT,get_us_packetsize(adsl_us_cellrate));
-	    set_packetsize(dev,EP_ATM_ISO_IN | USB_DIR_IN,get_ds_packetsize(adsl_ds_cellrate));
-    } else {
-	    set_packetsize(dev,EP_OBC_ISO_OUT,128);
-	    set_packetsize(dev,EP_OBC_ISO_IN | USB_DIR_IN,128);
-	    set_packetsize(dev,EP_ATM_ISO_OUT,get_us_packetsize(adsl_us_cellrate));
-	    set_packetsize(dev,EP_ATM_ISO_IN | USB_DIR_IN,get_ds_packetsize(adsl_ds_cellrate));
-    }
-}
 
 //======================================================================
 //	REPORTS ADSL status from the MSW
@@ -913,7 +750,6 @@ void setShowtime(void)
 		adsl_status = ADSL_STATUS_ATMREADY;
 		dev->recv_atm.cell_count = 0;
 		dev->send_atm.cell_count = 0;
-		adjust_bandwidth(dev,adsl_us_cellrate,adsl_ds_cellrate);
 		atm_start_snd(dev);
 		atm_start_rcv(dev);		
 		break;
@@ -940,9 +776,6 @@ void resetShowtime(void)
 		adsl_ds_cellrate = 0;
 		atm_stop_snd(dev);
 		atm_stop_rcv(dev);
-		if (!GlobalRemove) {
-			adjust_bandwidth(dev,adsl_us_cellrate,adsl_ds_cellrate);
-		}
 		break;
 	}
 }
@@ -1312,12 +1145,6 @@ static int start_device(struct unicorn_dev *dev,struct usb_device *usb_dev)
 	// Select alternate setting to match speed/bandwidth used
 	dev->alternate_setting = ALT_SETTING;
 
-	// patch the descriptors to use less bandwidth
-	set_packetsize(dev,EP_OBC_ISO_OUT,128);
-	set_packetsize(dev,EP_OBC_ISO_IN | USB_DIR_IN,128);
-	set_packetsize(dev,EP_ATM_ISO_OUT,0);
-	set_packetsize(dev,EP_ATM_ISO_IN | USB_DIR_IN,0);
-
 	if ((status = usb_set_interface (usb_dev, 0, dev->alternate_setting)) < 0) {
 		if (status == USB_ST_BANDWIDTH_ERROR) {
 			WARN("insufficient USB bandwidth\n");
@@ -1691,13 +1518,13 @@ ST_STATUS USB_init(
 	*CMDptrRd_I = (PWORD)(p + offsetof(USB_MEMORY,CmdBufRd_I));
 
 	ep_setting->ep0_size = dev->usb_dev->descriptor.bMaxPacketSize0;
-	ep_setting->ep1_size = dev->usb_dev->epmaxpacketin[EP_INTERRUPT];
-	ep_setting->ep2_size = dev->usb_dev->epmaxpacketout[EP_OBC_ISO_OUT];
-	ep_setting->ep3_size = dev->usb_dev->epmaxpacketin[EP_OBC_ISO_IN];
-	ep_setting->ep4_size = dev->usb_dev->epmaxpacketout[EP_ATM_ISO_OUT];
-	ep_setting->ep5_size = dev->usb_dev->epmaxpacketin[EP_ATM_ISO_IN];
-	ep_setting->ep6_size = dev->usb_dev->epmaxpacketout[EP_OBC_INT_OUT];
-	ep_setting->ep7_size = dev->usb_dev->epmaxpacketin[EP_OBC_INT_IN];
+	ep_setting->ep1_size = usb_maxpacket(dev->usb_dev,usb_rcvintpipe(dev->usb_dev,EP_INTERRUPT),0);
+	ep_setting->ep2_size = usb_maxpacket(dev->usb_dev,usb_sndisocpipe(dev->usb_dev,EP_OBC_ISO_OUT),1);
+	ep_setting->ep3_size = usb_maxpacket(dev->usb_dev,usb_rcvisocpipe(dev->usb_dev,EP_OBC_ISO_IN),0);
+	ep_setting->ep4_size = usb_maxpacket(dev->usb_dev,usb_sndisocpipe(dev->usb_dev,EP_ATM_ISO_OUT),1);
+	ep_setting->ep5_size = usb_maxpacket(dev->usb_dev,usb_rcvisocpipe(dev->usb_dev,EP_ATM_ISO_IN),0);
+	ep_setting->ep6_size = usb_maxpacket(dev->usb_dev,usb_sndintpipe(dev->usb_dev,EP_OBC_INT_OUT),1);
+	ep_setting->ep7_size = usb_maxpacket(dev->usb_dev,usb_rcvintpipe(dev->usb_dev,EP_OBC_INT_IN),0);
 	DBG(USB_D,"max packet size,ep0 %d,ep1 %d,ep2 %d,ep3 %d,ep4 %d,ep5 %d,ep6 %d,ep7 %d\n",
 	    ep_setting->ep0_size,ep_setting->ep1_size,ep_setting->ep2_size,ep_setting->ep3_size,
 	    ep_setting->ep4_size,ep_setting->ep5_size,ep_setting->ep6_size,ep_setting->ep7_size);
@@ -1807,7 +1634,8 @@ ST_STATUS USB_S_Write(T_ShortWrite *dataPtr,T_EpOut ep_out)
 			// Build ISOC descriptor
 			fill_isoc_urb(urb, dev->usb_dev, usb_sndisocpipe(dev->usb_dev,EP_OBC_ISO_OUT),
 				      dataPtr->cmdBuff, dataPtr->frameSize*sizeof(WORD),
-				      dev->usb_dev->epmaxpacketout[EP_OBC_ISO_OUT], 
+				      usb_maxpacket(dev->usb_dev,
+						    usb_sndisocpipe(dev->usb_dev,EP_OBC_ISO_OUT),1), 
 				      ObcWriteIsocComplete, dev);
 		} else {
 			// Int
@@ -1963,7 +1791,7 @@ ST_STATUS USB_Read(T_ReadData *dataPtr,T_EpOut ep_out,T_EpIn ep_in)
 			read_urb = dev->obc_iso_in;  
 			isoIn = TRUE;	
 			// Build ISOC descriptor
-			packet_size = dev->usb_dev->epmaxpacketin[ep];
+			packet_size = usb_maxpacket(dev->usb_dev,usb_rcvisocpipe(dev->usb_dev,ep),0);
 			fill_isoc_urb(read_urb, dev->usb_dev, usb_rcvisocpipe(dev->usb_dev,ep),
 				      ptr, dataPtr->rdSize*sizeof(WORD),
 				      packet_size, 
@@ -1989,7 +1817,8 @@ ST_STATUS USB_Read(T_ReadData *dataPtr,T_EpOut ep_out,T_EpIn ep_in)
 			write_urb = dev->obc_iso_out;
 		
 			// Build ISOC descriptor
-			packet_size = dev->usb_dev->epmaxpacketout[EP_OBC_ISO_OUT];
+			packet_size = usb_maxpacket(dev->usb_dev,
+						    usb_sndisocpipe(dev->usb_dev,EP_OBC_ISO_OUT),1);
 			fill_isoc_urb(write_urb, dev->usb_dev, usb_sndisocpipe(dev->usb_dev,EP_OBC_ISO_OUT),
 				      dataPtr->cmdBuff, dataPtr->wrSize*sizeof(WORD),
 				      packet_size, 
@@ -2261,6 +2090,8 @@ int unicorn_msw_control(struct unicorn_dev *dev,T_MswCtrl *ctrl)
 			}
 			info->Report = last_event;
 			info->Failure = last_failure;
+			info->RcvdCells = dev->recv_atm.cell_count;
+			info->SentCells = dev->send_atm.cell_count;
 		}
 		break;
 	case MSW_CTL_SET_DEBUG_LEVEL:
@@ -2444,6 +2275,7 @@ static struct usb_device_id unicorn_usb_ids[] = {
 	{ USB_DEVICE(0x07FA, 0xA904),driver_info: 70136}, // BeWAN
 	{ USB_DEVICE(0x07FA, 0xA905),driver_info: 70136}, // BeWAN
 	{ USB_DEVICE(0x07FA, 0x1012),driver_info: 20174}, // BeWAN
+	{ USB_DEVICE(0x135A, 0x0001),driver_info: 20174}, // urmet
 	{ }
 };
 
